{%- extends "base_templates/base.html" %}
{%- block main %}
      <div class="row">
        <div class="{{ config['GRID_CLASSES_ADMIN'] }}">
          {%- block content %}{% endblock %}
        </div>
      </div>
{%- endblock %}
